{
                                ********************************************
                                            I2C Low Level Routines     V1.0    
                                ********************************************
                                      coded by Beau Schwabe (Parallax)
                                ********************************************

}
VAR
    long     Flag, _SCL, _SDA, IOmask, i2cData  

PUB Initialize(SDA,SCL)
    Flag := 1
    _SCL := SCL
    _SDA := SDA
    IOmask := |<_SDA+ |<_SCL
    outa -=  outa & IOmask                              'Preset both IO's LOW 
    dira -=  dira & IOmask                              'Make both IO pins an INPUT
'----------------------------------------------------------------------------------------------------------------------------------------------
PUB i2cRead(ack,Bits)   
    if Flag == 1
       dira[_SDA]~                                      'Make SDA pin an INPUT          - Pullup resistor makes this HIGH
       repeat Bits 
         dira[_SCL]~                                    'Make SCL pin an INPUT          - Clock HIGH
         i2cData := (i2cData << 1) | ina[_SDA]          'Read SDA pin
         dira[_SCL]~~                                   'Drive SCL pin LOW              - Clock LOW
       dira[_SCL]~                                      'Make SCL pin an INPUT          - Clock HIGH
       dira[_SDA] := !ack                               'If ack = 0 then SDA becomes an Output otherwise it remains an Input
       dira[_SCL]~~                                     'Drive SCL pin LOW              - Clock LOW
       
       dira[_SDA]~                                      'Make SCL pin an INPUT          - Clock HIGH
       dira[_SDA]~~                                     'Drive SCL pin LOW              - Clock LOW
    return i2cData
'----------------------------------------------------------------------------------------------------------------------------------------------
PUB i2cWrite(Data,Bits)|ack,mask
    ack := 1 
    if Flag == 1
       mask := 1<<(Bits-1)
       repeat Bits
         dira[_SDA] := !((mask & Data)/mask)            'If the Data Bit = 1 then SDA becomes an Input (HIGH) otherwise it remains an Output (LOW)
         dira[_SCL]~                                    'Make SCL pin an INPUT          - Clock HIGH
         dira[_SCL]~~                                   'Drive SCL pin LOW              - Clock LOW
         mask := mask>>1                                'prepare Data mask for next bit
       dira[_SDA]~                                      'Make SDA pin an INPUT                          
       dira[_SCL]~                                      'Make SCL pin an INPUT          - Clock HIGH
       ack := ina[_SDA]                                 'Read SDA pin
       dira[_SCL]~~                                     'Drive SCL pin LOW              - Clock LOW
       dira[_SDA]~~                                     'Drive SDA pin LOW
     return ack
'----------------------------------------------------------------------------------------------------------------------------------------------
PUB i2cStart                    'SDA goes HIGH to LOW while SCL is HIGH
    dira -=  dira & IOmask                              'Make both IO pins an INPUT     - Pullup resistors make IO's HIGH
    waitpeq(IOmask,IOmask,0)                            'Wait for both pins to go HIGH
    dira[_SDA]~~                                        'Drive SDA pin LOW
    dira[_SCL]~~                                        'Drive SCL pin LOW
'----------------------------------------------------------------------------------------------------------------------------------------------
PUB i2cStop                     'SDA goes LOW to HIGH while SCL is HIGH
    dira[_SCL]~                                         'Make SCL pin an INPUT          - Pullup resistor makes this HIGH
    dira[_SDA]~                                         'Make SDA pin an INPUT          - Pullup resistor makes this HIGH
                                'Fixes Glitch in Read Mode during abnormal termination
    dira[_SCL]~~                                        'Drive SCL pin LOW
    dira[_SCL]~                                         'Make SCL pin an INPUT          - Pullup resistor makes this HIGH
'----------------------------------------------------------------------------------------------------------------------------------------------
PUB devicePresent(deviceAddress)|ack
    ack := 1
    if Flag == 1
       i2cStart
       ack := i2cWrite(deviceAddress | 0,8)
       i2cStop
    if testBus <> 3
       ack := 1
    ack := ( 1 - ack )* -1 + ack                        'Convert "0" to "-1" ; leave "1" alone
    return ack
'----------------------------------------------------------------------------------------------------------------------------------------------
PUB testBus
    dira -=  dira & IOmask                              'Make both IO pins an INPUT     - Pullup resistors make IO's HIGH
    return  ina[_SCL]<<1+ina[_SDA]